/*	$NetBSD: lfs_extern.h,v 1.13.2.2 2000/01/20 21:01:33 he Exp $	*/

/*-
 * Copyright (c) 1999 The NetBSD Foundation, Inc.
 * All rights reserved.
 *
 * This code is derived from software contributed to The NetBSD Foundation
 * by Konrad E. Schroder <perseant@hhhh.org>.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the NetBSD
 *      Foundation, Inc. and its contributors.
 * 4. Neither the name of The NetBSD Foundation nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE NETBSD FOUNDATION, INC. AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FOUNDATION OR CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
/*-
 * Copyright (c) 1991, 1993, 1994
 *	The Regents of the University of California.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *	This product includes software developed by the University of
 *	California, Berkeley and its contributors.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *	@(#)lfs_extern.h	8.6 (Berkeley) 5/8/95
 */

/*
 * Sysctl values for LFS.
 */
#define LFS_WRITEINDIR	 1 /* flush indirect blocks on non-checkpoint writes */
#define LFS_CLEAN_VNHEAD 2 /* put prev unrefed cleaned vnodes on head of free list */
#define LFS_DOSTATS      3
#define LFS_STATS        4
#define LFS_MAXID	 5

#define LFS_NAMES { \
	{ 0, 0 }, \
	{ "flushindir", CTLTYPE_INT }, \
	{ "clean_vnhead", CTLTYPE_INT }, \
	{ "dostats", CTLTYPE_INT }, \
	{ "stats", CTLTYPE_STRUCT }, \
}

struct fid;
struct mount;
struct nameidata;
struct proc;
struct statfs;
struct timeval;
struct inode;
struct uio;
struct mbuf;
struct dinode;
struct buf;
struct vnode;
struct lfs;
struct segment;
struct ucred;

extern struct pool mdlfs_inode_pool;		/* memory pool for inodes */

__BEGIN_DECLS
/* mdlfs_alloc.c */
int mdlfs_vcreate __P((struct mount *, ino_t, struct vnode **));


/* mdlfs_balloc.c */
int mdlfs_balloc __P((struct vnode *, int, u_long, ufs_daddr_t, struct buf **));

/* mdlfs_bio.c */
int mdlfs_bwrite_ext __P((struct buf *, int));
void mdlfs_flush_fs __P((struct mount *, int));
void mdlfs_flush __P((struct lfs *, int));
int mdlfs_check __P((struct vnode *, ufs_daddr_t, int));
void mdlfs_freebuf __P((struct buf *));
void mdlfs_countlocked __P((int *, long *));

/* mdlfs_cksum.c */
u_long mdlfs_cksum __P((void *, size_t));
u_long mdlfs_sb_cksum __P((struct dlfs *));

/* mdlfs_debug.c */
#ifdef DEBUG
void mdlfs_dump_super __P((struct lfs *));
void mdlfs_dump_dinode __P((struct dinode *));
void mdlfs_check_bpp __P((struct lfs *, struct segment *, char *, int));
void mdlfs_check_segsum __P((struct lfs *, struct segment *, char *, int));
#endif /* DEBUG */

/* mdlfs_inode.c */
void mdlfs_init __P((void));
struct dinode *mdlfs_ifind __P((struct lfs *, ino_t, struct buf *));

/* mdlfs_segment.c */
void mdlfs_imtime __P((struct lfs *));
int mdlfs_vflush __P((struct vnode *));
int mdlfs_writevnodes __P((struct lfs *, struct mount *, struct segment *, int));
int mdlfs_segwrite __P((struct mount *, int));
void mdlfs_writefile __P((struct lfs *, struct segment *, struct vnode *));
int mdlfs_writeinode __P((struct lfs *, struct segment *, struct inode *));
int mdlfs_gatherblock __P((struct segment *, struct buf *, int *));
int mdlfs_gather __P((struct lfs *, struct segment *, struct vnode *, int (*match )__P ((struct lfs *, struct buf *))));
void mdlfs_updatemeta __P((struct segment *));
int mdlfs_initseg __P((struct lfs *));
void mdlfs_newseg __P((struct lfs *));
int mdlfs_writeseg __P((struct lfs *, struct segment *));
void mdlfs_writesuper __P((struct lfs *, daddr_t));
int mdlfs_match_data __P((struct lfs *, struct buf *));
int mdlfs_match_indir __P((struct lfs *, struct buf *));
int mdlfs_match_dindir __P((struct lfs *, struct buf *));
int mdlfs_match_tindir __P((struct lfs *, struct buf *));
struct buf *mdlfs_newbuf __P((struct vnode *, ufs_daddr_t, size_t));
void mdlfs_callback __P((struct buf *));
void mdlfs_supercallback __P((struct buf *));
void mdlfs_shellsort __P((struct buf **, ufs_daddr_t *, int));
int mdlfs_vref __P((struct vnode *));
void mdlfs_vunref __P((struct vnode *));
void mdlfs_vunref_head __P((struct vnode *));
#ifdef MDLFS
/*
void mdlfs_chooseNextSeg(struct lfs* fs);
 */
#endif MDLFS

/* mdlfs_subr.c */
void mdlfs_seglock __P((struct lfs *, unsigned long));
void mdlfs_segunlock __P((struct lfs *));

/* mdlfs_syscalls.c */
int mdlfs_fastvget __P((struct mount *, ino_t, ufs_daddr_t, struct vnode **, struct dinode *, int *));
struct buf *mdlfs_fakebuf __P((struct vnode *, int, size_t, caddr_t));

/* mdlfs_vfsops.c */
int mdlfs_mountroot __P((void));
int mdlfs_mount __P((struct mount *, const char *, void *, struct nameidata *, struct proc *));
int mdlfs_mountfs __P((struct vnode *, struct mount *, struct proc *));
int mdlfs_unmount __P((struct mount *, int, struct proc *));
int mdlfs_statfs __P((struct mount *, struct statfs *, struct proc *));
int mdlfs_sync __P((struct mount *, int, struct ucred *, struct proc *));
int mdlfs_vget __P((struct mount *, ino_t, struct vnode **));
int mdlfs_fhtovp __P((struct mount *, struct fid *, struct vnode **));
int mdlfs_vptofh __P((struct vnode *, struct fid *));
int mdlfs_sysctl __P((int *, u_int, void *, size_t *, void *, size_t,
		    struct proc *));


int mdlfs_valloc		__P((void *));
int mdlfs_vfree		__P((void *));
int mdlfs_bwrite		__P((void *));
int mdlfs_update		__P((void *));
int mdlfs_truncate	__P((void *));
int mdlfs_blkatoff	__P((void *));
int mdlfs_fsync		__P((void *));
int mdlfs_symlink		__P((void *));
int mdlfs_mknod		__P((void *));
int mdlfs_create		__P((void *));
int mdlfs_mkdir		__P((void *));
int mdlfs_remove		__P((void *));
int mdlfs_rmdir		__P((void *));
int mdlfs_link		__P((void *));
int mdlfs_rename		__P((void *));
int mdlfs_getattr		__P((void *));
int mdlfs_close		__P((void *));
int mdlfs_inactive	__P((void *));
int mdlfs_reclaim		__P((void *));
int mdlfs_whiteout	__P((void *));
int mdlfs_read		__P((void *));
int mdlfs_write		__P((void *));
int mdlfs_strategy	__P((void *));

/* mdlfs_remap.c */
#ifdef MDLFS
void mdlfs_remap(struct lfs* fs, struct buf* bp);
#endif MDLFS

__END_DECLS
extern int lfs_mount_type;
extern int (**mdlfs_vnodeop_p) __P((void *));
extern int (**mdlfs_specop_p) __P((void *));
extern int (**mdlfs_fifoop_p) __P((void *));

