/* 
 * file name : mdlfs_remap.c 
 * date      : 11/24/00
 */

#include <sys/param.h>
#include <sys/systm.h>
#include <sys/namei.h>
#include <sys/resourcevar.h>
#include <sys/kernel.h>
#include <sys/file.h>
#include <sys/stat.h>
#include <sys/buf.h>
#include <sys/proc.h>
#include <sys/conf.h>
#include <sys/mount.h>
#include <sys/vnode.h>
#include <sys/time.h>

#include <ufs/ufs/quota.h>
#include <ufs/ufs/inode.h>
#include <ufs/ufs/dir.h>
#include <ufs/ufs/ufsmount.h>
#include <ufs/ufs/ufs_extern.h>

#include <ufs/mdlfs/mdlfs.h>
#include <ufs/mdlfs/mdlfs_extern.h>

/* Macros to clear/set/test flags. */
# define	SET(t, f)	(t) |= (f)
# define	CLR(t, f)	(t) &= ~(f)
# define	ISSET(t, f)	((t) & (f))

void mdlfs_remap(struct lfs* fs, struct buf* bp) 
{

	ufs_daddr_t begin_block, end_block;
		/* begin and end blocks for a 'disk' */
	ufs_daddr_t newpbn, pbn ;
	int curdev;

#ifdef PRINT_FUNCS
printf("mdlfs_remap()\n");
#endif /* PRINT_FUNCS */

	pbn = bp->b_blkno;
		/* underlying physical block number spanning all disks*/

	for (curdev = 0; curdev < fs->mdlfs_devnum; curdev++)
	{
		begin_block = fs->mdlfs_devices[curdev].firstSeg;
		/* last block inlast segment */ 
		end_block   = fs->mdlfs_devices[curdev].lastSeg; 
		
		/* MAY want to out ifdef sanity checks by DIAGNOSTIC */

		/* move onto the next device? */
		if (pbn <= end_block) 
		{
			/* calc the newlbn with respect to this disk */ 
			newpbn = pbn - begin_block;
			break; 
		}
	}

	/* sanity check, make sure that pbn is not beyond the last block of the 
	 * last device
	 */
	if (curdev == fs->mdlfs_devnum)
	{
		/* panic ("block number in remap beyond valid range\n"); */
		printf("block number in remap beyond valid range\n");
		bp->b_blkno = -1;
	}
	   	
	/* modifying the physical blockno to disk specific value */
	if (curdev != 0) {

		/* assign the remapped block number */
		bp->b_blkno = newpbn;

	}
	/* remap the operation to the correct device */
	bp->b_dev = fs->mdlfs_devices[curdev].dev;
	
	if ((bp->b_flags & B_CALL) && !(bp->b_flags & B_READ)) {
		fs->mdlfs_devices[curdev].writeReqs++;
		SET(bp->b_flags, B_COUNT);
		microtime(&bp->opStartTime); 
	}

}

